@@ -62,6 +62,7 @@ query_axfr(struct nsd *nsd, struct query *query)
 			RCODE_SET(query->packet, RCODE_NOTAUTH);
 			return QUERY_PROCESSED;
 		}
+		ZTATUP(nsd, query->axfr_zone, raxfr);
 
 		query->axfr_current_domain = qdomain;
 		query->axfr_current_rrset = NULL;
@@ -175,10 +176,10 @@ answer_axfr_ixfr(struct nsd *nsd, struct query *q)
 			if(!zone_opt ||
 			   acl_check_incoming(zone_opt->pattern->provide_xfr, q, &acl)==-1)
 			{
-				if (verbosity > 0) {
+				if (verbosity >= 2) {
 					char a[128];
 					addr2str(&q->addr, a, sizeof(a));
-					VERBOSITY(1, (LOG_INFO, "axfr for zone %s from client %s refused, %s",
+					VERBOSITY(2, (LOG_INFO, "axfr for %s from %s refused, %s",
 						dname_to_string(q->qname, NULL), a, acl?"blocked":"no acl matches"));
 				}
 				DEBUG(DEBUG_XFRD,1, (LOG_INFO, "axfr refused, %s",
