@@ -156,8 +156,8 @@ read_rrset(udb_base* udb, namedb_type* db, zone_type* zone,
 	rrset = (rrset_type *) region_alloc(db->region, sizeof(rrset_type));
 	rrset->zone = zone;
 	rrset->rr_count = calculate_rr_count(udb, urrset);
-	rrset->rrs = (rr_type *) region_alloc(
-		db->region, rrset->rr_count * sizeof(rr_type));
+	rrset->rrs = (rr_type *) region_alloc_array(
+		db->region, rrset->rr_count, sizeof(rr_type));
 	/* add the RRs */
 	udb_ptr_new(&urr, udb, &RRSET(urrset)->rrs);
 	for(i=0; i<rrset->rr_count; i++) {
@@ -272,6 +272,7 @@ namedb_zone_create(namedb_type* db, const dname_type* dname,
 	zone->filename = NULL;
 	zone->logstr = NULL;
 	zone->mtime = 0;
+	zone->zonestatid = 0;
 	zone->is_secure = 0;
 	zone->is_changed = 0;
 	zone->is_ok = 1;
@@ -606,7 +607,7 @@ namedb_read_zonefile(struct nsd* nsd, struct zone* zone, udb_base* taskudb,
 			zone->logstr = NULL;
 		}
 	} else {
-		VERBOSITY(1, (LOG_INFO, "zone %s read with no errors",
+		VERBOSITY(1, (LOG_INFO, "zone %s read with success",
 			zone->opts->name));
 		zone->is_ok = 1;
 		zone->is_changed = 0;
