@@ -217,7 +217,7 @@ static inline size_t
 dname_total_size(const dname_type *dname)
 {
 	return (sizeof(dname_type)
-		+ ((dname->label_count + dname->name_size)
+		+ ((((size_t)dname->label_count) + ((size_t)dname->name_size))
 		   * sizeof(uint8_t)));
 }
 
