@@ -1,7 +1,7 @@
 /*
  * region-allocator.h -- region based memory allocator.
  *
- * Copyright (c) 2001-2011, NLnet Labs. All rights reserved.
+ * Copyright (c) 2001-2006, NLnet Labs. All rights reserved.
  *
  * See LICENSE for the license.
  *
@@ -88,6 +88,8 @@ void region_remove_cleanup(region_type *region,
  */
 void *region_alloc(region_type *region, size_t size);
 
+/** Allocate array with integer overflow checks, in region */
+void *region_alloc_array(region_type *region, size_t num, size_t size);
 
 /*
  * Allocate SIZE bytes of memory inside REGION and copy INIT into it.
@@ -96,6 +98,12 @@ void *region_alloc(region_type *region, size_t size);
  */
 void *region_alloc_init(region_type *region, const void *init, size_t size);
 
+/** 
+ * Allocate array (with integer overflow check on sizes), and init with
+ * the given array copied into it.  Allocated in the region
+ */
+void *region_alloc_array_init(region_type *region, const void *init,
+	size_t num, size_t size);
 
 /*
  * Allocate SIZE bytes of memory inside REGION that are initialized to
@@ -104,6 +112,11 @@ void *region_alloc_init(region_type *region, const void *init, size_t size);
  */
 void *region_alloc_zero(region_type *region, size_t size);
 
+/** 
+ * Allocate array (with integer overflow check on sizes), and zero it.
+ * Allocated in the region.
+ */
+void *region_alloc_array_zero(region_type *region, size_t num, size_t size);
 
 /*
  * Run the cleanup actions and free all memory associated with REGION.
@@ -129,6 +142,10 @@ void region_dump_stats(region_type *region, FILE *out);
 
 /* get size of recyclebin */
 size_t region_get_recycle_size(region_type* region);
+/* get size of region memory in use */
+size_t region_get_mem(region_type* region);
+/* get size of region memory unused */
+size_t region_get_mem_unused(region_type* region);
 
 /* Debug print REGION statistics to LOG. */
 void region_log_stats(region_type *region);
