@@ -29,11 +29,13 @@
 #include <time.h>
 #include <unistd.h>
 #include <signal.h>
-#include <fcntl.h>
 #include <netdb.h>
 #ifndef SHUT_WR
 #define SHUT_WR 1
 #endif
+#ifdef HAVE_MMAP
+#include <sys/mman.h>
+#endif /* HAVE_MMAP */
 #include <openssl/rand.h>
 #ifndef USE_MINI_EVENT
 #  ifdef HAVE_EVENT_H
@@ -339,6 +341,180 @@ static void set_bind8_alarm(struct nsd* nsd)
 }
 #endif
 
+/* set zone stat ids for zones initially read in */
+static void
+zonestatid_tree_set(struct nsd* nsd)
+{
+	struct radnode* n;
+	for(n=radix_first(nsd->db->zonetree); n; n=radix_next(n)) {
+		zone_type* zone = (zone_type*)n->elem;
+		zone->zonestatid = getzonestatid(nsd->options, zone->opts);
+	}
+}
+
+#ifdef USE_ZONE_STATS
+void
+server_zonestat_alloc(struct nsd* nsd)
+{
+	size_t num = (nsd->options->zonestatnames->count==0?1:
+			nsd->options->zonestatnames->count);
+	size_t sz = sizeof(struct nsdst)*num;
+	char tmpfile[256];
+	uint8_t z = 0;
+
+	/* file names */
+	nsd->zonestatfname[0] = 0;
+	nsd->zonestatfname[1] = 0;
+	snprintf(tmpfile, sizeof(tmpfile), "%snsd.%u.zstat.0",
+		nsd->options->xfrdir, (unsigned)getpid());
+	nsd->zonestatfname[0] = region_strdup(nsd->region, tmpfile);
+	snprintf(tmpfile, sizeof(tmpfile), "%snsd.%u.zstat.1",
+		nsd->options->xfrdir, (unsigned)getpid());
+	nsd->zonestatfname[1] = region_strdup(nsd->region, tmpfile);
+
+	/* file descriptors */
+	nsd->zonestatfd[0] = open(nsd->zonestatfname[0], O_CREAT|O_RDWR, 0600);
+	if(nsd->zonestatfd[0] == -1) {
+		log_msg(LOG_ERR, "cannot create %s: %s", nsd->zonestatfname[0],
+			strerror(errno));
+		exit(1);
+	}
+	nsd->zonestatfd[1] = open(nsd->zonestatfname[1], O_CREAT|O_RDWR, 0600);
+	if(nsd->zonestatfd[0] == -1) {
+		log_msg(LOG_ERR, "cannot create %s: %s", nsd->zonestatfname[1],
+			strerror(errno));
+		close(nsd->zonestatfd[0]);
+		unlink(nsd->zonestatfname[0]);
+		exit(1);
+	}
+
+#ifdef HAVE_MMAP
+	if(lseek(nsd->zonestatfd[0], (off_t)sz-1, SEEK_SET) == -1) {
+		log_msg(LOG_ERR, "lseek %s: %s", nsd->zonestatfname[0],
+			strerror(errno));
+		exit(1);
+	}
+	if(write(nsd->zonestatfd[0], &z, 1) == -1) {
+		log_msg(LOG_ERR, "cannot extend stat file %s (%s)",
+			nsd->zonestatfname[0], strerror(errno));
+		exit(1);
+	}
+	if(lseek(nsd->zonestatfd[1], (off_t)sz-1, SEEK_SET) == -1) {
+		log_msg(LOG_ERR, "lseek %s: %s", nsd->zonestatfname[1],
+			strerror(errno));
+		exit(1);
+	}
+	if(write(nsd->zonestatfd[1], &z, 1) == -1) {
+		log_msg(LOG_ERR, "cannot extend stat file %s (%s)",
+			nsd->zonestatfname[1], strerror(errno));
+		exit(1);
+	}
+	nsd->zonestat[0] = (struct nsdst*)mmap(NULL, sz, PROT_READ|PROT_WRITE,
+		MAP_SHARED, nsd->zonestatfd[0], 0);
+	if(nsd->zonestat[0] == MAP_FAILED) {
+		log_msg(LOG_ERR, "mmap failed: %s", strerror(errno));
+		unlink(nsd->zonestatfname[0]);
+		unlink(nsd->zonestatfname[1]);
+		exit(1);
+	}
+	nsd->zonestat[1] = (struct nsdst*)mmap(NULL, sz, PROT_READ|PROT_WRITE,
+		MAP_SHARED, nsd->zonestatfd[1], 0);
+	if(nsd->zonestat[1] == MAP_FAILED) {
+		log_msg(LOG_ERR, "mmap failed: %s", strerror(errno));
+		unlink(nsd->zonestatfname[0]);
+		unlink(nsd->zonestatfname[1]);
+		exit(1);
+	}
+	memset(nsd->zonestat[0], 0, sz);
+	memset(nsd->zonestat[1], 0, sz);
+	nsd->zonestatsize[0] = num;
+	nsd->zonestatsize[1] = num;
+	nsd->zonestatdesired = num;
+	nsd->zonestatsizenow = num;
+	nsd->zonestatnow = nsd->zonestat[0];
+#endif /* HAVE_MMAP */
+}
+
+void
+zonestat_remap(struct nsd* nsd, int idx, size_t sz)
+{
+#ifdef HAVE_MMAP
+#ifdef MREMAP_MAYMOVE
+	nsd->zonestat[idx] = (struct nsdst*)mremap(nsd->zonestat[idx],
+		sizeof(struct nsdst)*nsd->zonestatsize[idx], sz,
+		MREMAP_MAYMOVE);
+	if(nsd->zonestat[idx] == MAP_FAILED) {
+		log_msg(LOG_ERR, "mremap failed: %s", strerror(errno));
+		exit(1);
+	}
+#else /* !HAVE MREMAP */
+	if(msync(nsd->zonestat[idx],
+		sizeof(struct nsdst)*nsd->zonestatsize[idx], MS_ASYNC) != 0)
+		log_msg(LOG_ERR, "msync failed: %s", strerror(errno));
+	if(munmap(nsd->zonestat[idx],
+		sizeof(struct nsdst)*nsd->zonestatsize[idx]) != 0)
+		log_msg(LOG_ERR, "munmap failed: %s", strerror(errno));
+	nsd->zonestat[idx] = (struct nsdst*)mmap(NULL, sz,
+		PROT_READ|PROT_WRITE, MAP_SHARED, nsd->zonestatfd[idx], 0);
+	if(nsd->zonestat[idx] == MAP_FAILED) {
+		log_msg(LOG_ERR, "mmap failed: %s", strerror(errno));
+		exit(1);
+	}
+#endif /* MREMAP */
+#endif /* HAVE_MMAP */
+}
+
+/* realloc the zonestat array for the one that is not currently in use,
+ * to match the desired new size of the array (if applicable) */
+void
+server_zonestat_realloc(struct nsd* nsd)
+{
+#ifdef HAVE_MMAP
+	uint8_t z = 0;
+	size_t sz;
+	int idx = 0; /* index of the zonestat array that is not in use */
+	if(nsd->zonestatnow == nsd->zonestat[0])
+		idx = 1;
+	if(nsd->zonestatsize[idx] == nsd->zonestatdesired)
+		return;
+	sz = sizeof(struct nsdst)*nsd->zonestatdesired;
+	if(lseek(nsd->zonestatfd[idx], (off_t)sz-1, SEEK_SET) == -1) {
+		log_msg(LOG_ERR, "lseek %s: %s", nsd->zonestatfname[idx],
+			strerror(errno));
+		exit(1);
+	}
+	if(write(nsd->zonestatfd[idx], &z, 1) == -1) {
+		log_msg(LOG_ERR, "cannot extend stat file %s (%s)",
+			nsd->zonestatfname[idx], strerror(errno));
+		exit(1);
+	}
+	zonestat_remap(nsd, idx, sz);
+	/* zero the newly allocated region */
+	if(nsd->zonestatdesired > nsd->zonestatsize[idx]) {
+		memset(((char*)nsd->zonestat[idx])+sizeof(struct nsdst) *
+			nsd->zonestatsize[idx], 0, sizeof(struct nsdst) *
+			(nsd->zonestatdesired - nsd->zonestatsize[idx]));
+	}
+	nsd->zonestatsize[idx] = nsd->zonestatdesired;
+#endif /* HAVE_MMAP */
+}
+
+/* switchover to use the other array for the new children, that
+ * briefly coexist with the old children.  And we want to avoid them
+ * both writing to the same statistics arrays. */
+void
+server_zonestat_switch(struct nsd* nsd)
+{
+	if(nsd->zonestatnow == nsd->zonestat[0]) {
+		nsd->zonestatnow = nsd->zonestat[1];
+		nsd->zonestatsizenow = nsd->zonestatsize[1];
+	} else {
+		nsd->zonestatnow = nsd->zonestat[0];
+		nsd->zonestatsizenow = nsd->zonestatsize[0];
+	}
+}
+#endif /* USE_ZONE_STATS */
+
 static void
 cleanup_dname_compression_tables(void *ptr)
 {
@@ -359,8 +535,8 @@ initialize_dname_compression_tables(struct nsd *nsd)
 				compressed_dname_offsets);
 			free(compressed_dname_offsets);
 		}
-		compressed_dname_offsets = (uint16_t *) xalloc(
-			needed * sizeof(uint16_t));
+		compressed_dname_offsets = (uint16_t *) xmallocarray(
+			needed, sizeof(uint16_t));
 		region_add_cleanup(nsd->db->region, cleanup_dname_compression_tables,
 			compressed_dname_offsets);
 		compression_table_capacity = needed;
@@ -673,6 +849,10 @@ server_prepare(struct nsd *nsd)
 			nsd->dbfile, strerror(errno));
 		unlink(nsd->task[0]->fname);
 		unlink(nsd->task[1]->fname);
+#ifdef USE_ZONE_STATS
+		unlink(nsd->zonestatfname[0]);
+		unlink(nsd->zonestatfname[1]);
+#endif
 		xfrd_del_tempdir(nsd);
 		return -1;
 	}
@@ -682,6 +862,7 @@ server_prepare(struct nsd *nsd)
 	if(nsd->options->zonefiles_check || (nsd->options->database == NULL ||
 		nsd->options->database[0] == 0))
 		namedb_check_zonefiles(nsd, nsd->options, NULL, NULL);
+	zonestatid_tree_set(nsd);
 
 	compression_table_capacity = 0;
 	initialize_dname_compression_tables(nsd);
@@ -907,6 +1088,10 @@ server_send_soa_xfrd(struct nsd* nsd, int shortsoa)
 			unlinkpid(nsd->pidfile);
 			unlink(nsd->task[0]->fname);
 			unlink(nsd->task[1]->fname);
+#ifdef USE_ZONE_STATS
+			unlink(nsd->zonestatfname[0]);
+			unlink(nsd->zonestatfname[1]);
+#endif
 			/* write the nsd.db to disk, wait for it to complete */
 			udb_base_sync(nsd->db->udb, 1);
 			udb_base_close(nsd->db->udb);
@@ -1148,6 +1333,10 @@ server_reload(struct nsd *nsd, region_type* server_region, netio_type* netio,
 	time(&nsd->st.boot);
 	set_bind8_alarm(nsd);
 #endif
+#ifdef USE_ZONE_STATS
+	server_zonestat_realloc(nsd); /* realloc for new children */
+	server_zonestat_switch(nsd);
+#endif
 
 	/* listen for the signals of failed children again */
 	sigaction(SIGCHLD, &old_sigchld, NULL);
@@ -1200,6 +1389,9 @@ server_reload(struct nsd *nsd, region_type* server_region, netio_type* netio,
 #endif
 	udb_ptr_unlink(&last_task, nsd->task[nsd->mytask]);
 	task_process_sync(nsd->task[nsd->mytask]);
+#ifdef USE_ZONE_STATS
+	server_zonestat_realloc(nsd); /* realloc for next children */
+#endif
 
 	/* send soainfo to the xfrd process, signal it that reload is done,
 	 * it picks up the taskudb */
@@ -1547,6 +1739,10 @@ server_main(struct nsd *nsd)
 	unlinkpid(nsd->pidfile);
 	unlink(nsd->task[0]->fname);
 	unlink(nsd->task[1]->fname);
+#ifdef USE_ZONE_STATS
+	unlink(nsd->zonestatfname[0]);
+	unlink(nsd->zonestatfname[1]);
+#endif
 
 	if(reload_listener.fd != -1) {
 		sig_atomic_t cmd = NSD_QUIT;
@@ -1733,8 +1929,9 @@ server_child(struct nsd *nsd)
 	 * connections.
 	 */
 	tcp_accept_handler_count = nsd->ifs;
-	tcp_accept_handlers = (struct tcp_accept_handler_data*) region_alloc(
-		server_region, nsd->ifs * sizeof(*tcp_accept_handlers));
+	tcp_accept_handlers = (struct tcp_accept_handler_data*)
+		region_alloc_array(server_region,
+		nsd->ifs, sizeof(*tcp_accept_handlers));
 	if (nsd->server_kind & NSD_SERVER_TCP) {
 		for (i = 0; i < nsd->ifs; ++i) {
 			struct event *handler = &tcp_accept_handlers[i].event;
@@ -1831,6 +2028,7 @@ handle_udp(int fd, short event, void* arg)
 		if (errno != EAGAIN && errno != EINTR) {
 			log_msg(LOG_ERR, "recvmmsg failed: %s", strerror(errno));
 			STATUP(data->nsd, rxerr);
+			/* No zone statup */
 		}
 		/* Simply no data available */
 		return;
@@ -1843,17 +2041,20 @@ handle_udp(int fd, short event, void* arg)
 			log_msg(LOG_ERR, "recvmmsg %d failed %s", i, strerror(
 				msgs[i].msg_hdr.msg_flags));
 			STATUP(data->nsd, rxerr);
+			/* No zone statup */
 			query_reset(queries[i], UDP_MAX_MESSAGE_LEN, 0);
 			iovecs[i].iov_len = buffer_remaining(q->packet);
 			goto swap_drop;
 		}
 
 		/* Account... */
+#ifdef BIND8_STATS
 		if (data->socket->addr->ai_family == AF_INET) {
 			STATUP(data->nsd, qudp);
 		} else if (data->socket->addr->ai_family == AF_INET6) {
 			STATUP(data->nsd, qudp6);
 		}
+#endif
 
 		buffer_skip(q->packet, received);
 		buffer_flip(q->packet);
@@ -1862,8 +2063,17 @@ handle_udp(int fd, short event, void* arg)
 		if (server_process_query_udp(data->nsd, q) != QUERY_DISCARDED) {
 			if (RCODE(q->packet) == RCODE_OK && !AA(q->packet)) {
 				STATUP(data->nsd, nona);
+				ZTATUP(data->nsd, q->zone, nona);
 			}
 
+#ifdef USE_ZONE_STATS
+			if (data->socket->addr->ai_family == AF_INET) {
+				ZTATUP(data->nsd, q->zone, qudp);
+			} else if (data->socket->addr->ai_family == AF_INET6) {
+				ZTATUP(data->nsd, q->zone, qudp6);
+			}
+#endif
+
 			/* Add EDNS0 and TSIG info if necessary.  */
 			query_add_optional(q, data->nsd);
 
@@ -1872,14 +2082,18 @@ handle_udp(int fd, short event, void* arg)
 #ifdef BIND8_STATS
 			/* Account the rcode & TC... */
 			STATUP2(data->nsd, rcode, RCODE(q->packet));
-			if (TC(q->packet))
+			ZTATUP2(data->nsd, q->zone, rcode, RCODE(q->packet));
+			if (TC(q->packet)) {
 				STATUP(data->nsd, truncated);
+				ZTATUP(data->nsd, q->zone, truncated);
+			}
 #endif /* BIND8_STATS */
 		} else {
 			query_reset(queries[i], UDP_MAX_MESSAGE_LEN, 0);
 			iovecs[i].iov_len = buffer_remaining(q->packet);
 		swap_drop:
 			STATUP(data->nsd, dropped);
+			ZTATUP(data->nsd, q->zone, dropped);
 			if(i != recvcount-1) {
 				/* swap with last and decrease recvcount */
 				struct mmsghdr mtmp = msgs[i];
@@ -1950,6 +2164,7 @@ handle_udp(int fd, short event, void* arg)
 		if (errno != EAGAIN && errno != EINTR) {
 			log_msg(LOG_ERR, "recvmmsg failed: %s", strerror(errno));
 			STATUP(data->nsd, rxerr);
+			/* No zone statup */
 		}
 		/* Simply no data available */
 		return;
@@ -1960,6 +2175,7 @@ handle_udp(int fd, short event, void* arg)
 		if (received == -1) {
 			log_msg(LOG_ERR, "recvmmsg failed");
 			STATUP(data->nsd, rxerr);
+			/* No zone statup */
 			/* the error can be found in msgs[i].msg_hdr.msg_flags */
 			query_reset(queries[i], UDP_MAX_MESSAGE_LEN, 0);
 			continue;
@@ -1984,6 +2200,7 @@ handle_udp(int fd, short event, void* arg)
 			if (errno != EAGAIN && errno != EINTR) {
 				log_msg(LOG_ERR, "recvfrom failed: %s", strerror(errno));
 				STATUP(data->nsd, rxerr);
+				/* No zone statup */
 			}
 			return;
 		}
@@ -2003,8 +2220,17 @@ handle_udp(int fd, short event, void* arg)
 		if (server_process_query_udp(data->nsd, q) != QUERY_DISCARDED) {
 			if (RCODE(q->packet) == RCODE_OK && !AA(q->packet)) {
 				STATUP(data->nsd, nona);
+				ZTATUP(data->nsd, q->zone, nona);
 			}
 
+#ifdef USE_ZONE_STATS
+			if (data->socket->addr->ai_family == AF_INET) {
+				ZTATUP(data->nsd, q->zone, qudp);
+			} else if (data->socket->addr->ai_family == AF_INET6) {
+				ZTATUP(data->nsd, q->zone, qudp6);
+			}
+#endif
+
 			/* Add EDNS0 and TSIG info if necessary.  */
 			query_add_optional(q, data->nsd);
 
@@ -2022,18 +2248,23 @@ handle_udp(int fd, short event, void* arg)
 				addr2str(&q->addr, a, sizeof(a));
 				log_msg(LOG_ERR, "sendto %s failed: %s", a, es);
 				STATUP(data->nsd, txerr);
+				ZTATUP(data->nsd, q->zone, txerr);
 			} else if ((size_t) sent != buffer_remaining(q->packet)) {
 				log_msg(LOG_ERR, "sent %d in place of %d bytes", sent, (int) buffer_remaining(q->packet));
 			} else {
 #ifdef BIND8_STATS
 				/* Account the rcode & TC... */
 				STATUP2(data->nsd, rcode, RCODE(q->packet));
-				if (TC(q->packet))
+				ZTATUP2(data->nsd, q->zone, rcode, RCODE(q->packet));
+				if (TC(q->packet)) {
 					STATUP(data->nsd, truncated);
+					ZTATUP(data->nsd, q->zone, truncated);
+				}
 #endif /* BIND8_STATS */
 			}
 		} else {
 			STATUP(data->nsd, dropped);
+			ZTATUP(data->nsd, q->zone, dropped);
 		}
 #ifndef NONBLOCKING_IS_BROKEN
 #ifdef HAVE_RECVMMSG
@@ -2202,8 +2433,9 @@ handle_tcp_reading(int fd, short event, void* arg)
 	assert(buffer_position(data->query->packet) == data->query->tcplen);
 
 	/* Account... */
+#ifdef BIND8_STATS
 #ifndef INET6
-        STATUP(data->nsd, ctcp);
+	STATUP(data->nsd, ctcp);
 #else
 	if (data->query->addr.ss_family == AF_INET) {
 		STATUP(data->nsd, ctcp);
@@ -2211,6 +2443,7 @@ handle_tcp_reading(int fd, short event, void* arg)
 		STATUP(data->nsd, ctcp6);
 	}
 #endif
+#endif /* BIND8_STATS */
 
 	/* We have a complete query, process it.  */
 
@@ -2222,15 +2455,31 @@ handle_tcp_reading(int fd, short event, void* arg)
 	if (data->query_state == QUERY_DISCARDED) {
 		/* Drop the packet and the entire connection... */
 		STATUP(data->nsd, dropped);
+		ZTATUP(data->nsd, data->query->zone, dropped);
 		cleanup_tcp_handler(data);
 		return;
 	}
 
+#ifdef BIND8_STATS
 	if (RCODE(data->query->packet) == RCODE_OK
 	    && !AA(data->query->packet))
 	{
 		STATUP(data->nsd, nona);
+		ZTATUP(data->nsd, data->query->zone, nona);
 	}
+#endif /* BIND8_STATS */
+
+#ifdef USE_ZONE_STATS
+#ifndef INET6
+	ZTATUP(data->nsd, data->query->zone, ctcp);
+#else
+	if (data->query->addr.ss_family == AF_INET) {
+		ZTATUP(data->nsd, data->query->zone, ctcp);
+	} else if (data->query->addr.ss_family == AF_INET6) {
+		ZTATUP(data->nsd, data->query->zone, ctcp6);
+	}
+#endif
+#endif /* USE_ZONE_STATS */
 
 	query_add_optional(data->query, data->nsd);
 
@@ -2523,10 +2772,18 @@ handle_tcp_accept(int fd, short event, void* arg)
 
 	event_set(&tcp_data->event, s, EV_PERSIST | EV_READ | EV_TIMEOUT,
 		handle_tcp_reading, tcp_data);
-	if(event_base_set(data->event.ev_base, &tcp_data->event) != 0)
-		log_msg(LOG_ERR, "cannot set tcp event base");
-	if(event_add(&tcp_data->event, &timeout) != 0)
+	if(event_base_set(data->event.ev_base, &tcp_data->event) != 0) {
 		log_msg(LOG_ERR, "cannot set tcp event base");
+		close(s);
+		region_destroy(tcp_region);
+		return;
+	}
+	if(event_add(&tcp_data->event, &timeout) != 0) {
+		log_msg(LOG_ERR, "cannot add tcp to event base");
+		close(s);
+		region_destroy(tcp_region);
+		return;
+	}
 
 	/*
 	 * Keep track of the total number of TCP handlers installed so
