@@ -258,6 +258,18 @@ xalloc(size_t size)
 	return result;
 }
 
+void *
+xmallocarray(size_t num, size_t size)
+{  
+        void *result = reallocarray(NULL, num, size);
+   
+        if (!result) {
+                log_msg(LOG_ERR, "reallocarray failed: %s", strerror(errno));
+                exit(1);
+        }
+        return result;
+}
+
 void *
 xalloc_zero(size_t size)
 {
@@ -269,6 +281,17 @@ xalloc_zero(size_t size)
 	return result;
 }
 
+void *
+xalloc_array_zero(size_t num, size_t size)
+{
+	void *result = calloc(num, size);
+	if (!result) {
+		log_msg(LOG_ERR, "calloc failed: %s", strerror(errno));
+		exit(1);
+	}
+	return result;
+}
+
 void *
 xrealloc(void *ptr, size_t size)
 {
@@ -553,7 +576,8 @@ hex_pton(const char* src, uint8_t* target, size_t targsize)
 		return -1;
 	}
 	while(*src) {
-		if(!isxdigit((int)src[0]) || !isxdigit((int)src[1]))
+		if(!isxdigit((unsigned char)src[0]) ||
+			!isxdigit((unsigned char)src[1]))
 			return -1;
 		*t++ = hexdigit_to_int(src[0]) * 16 +
 			hexdigit_to_int(src[1]) ;
@@ -658,7 +682,7 @@ b32_pton(const char *src, uint8_t *target, size_t tsize)
 		if(p+5 >= tsize*8)
 		       return -1;
 
-		if(isspace(ch))
+		if(isspace((unsigned char)ch))
 			continue;
 
 		if(ch >= '0' && ch <= '9')
@@ -690,13 +714,13 @@ strip_string(char *str)
 	char *start = str;
 	char *end = str + strlen(str) - 1;
 
-	while (isspace(*start))
+	while (isspace((unsigned char)*start))
 		++start;
 	if (start > end) {
 		/* Completely blank. */
 		str[0] = '\0';
 	} else {
-		while (isspace(*end))
+		while (isspace((unsigned char)*end))
 			--end;
 		*++end = '\0';
 
