@@ -135,7 +135,9 @@ lookup_table_type *lookup_by_id(lookup_table_type table[], int id);
  * return NULL.
  */
 void *xalloc(size_t size);
+void *xmallocarray(size_t num, size_t size);
 void *xalloc_zero(size_t size);
+void *xalloc_array_zero(size_t num, size_t size);
 void *xrealloc(void *ptr, size_t size);
 
 /*
