@@ -103,7 +103,7 @@ zparser_conv_hex(region_type *region, const char *hex, size_t len)
 		while (*hex) {
 			*t = 0;
 			for (i = 16; i >= 1; i -= 15) {
-				if (isxdigit((int)*hex)) {
+				if (isxdigit((unsigned char)*hex)) {
 					*t += hexdigit_to_int(*hex) * i;
 				} else {
 					zc_error_prev_line(
@@ -144,7 +144,7 @@ zparser_conv_hex_length(region_type *region, const char *hex, size_t len)
 		while (*hex) {
 			*t = 0;
 			for (i = 16; i >= 1; i -= 15) {
-				if (isxdigit((int)*hex)) {
+				if (isxdigit((unsigned char)*hex)) {
 					*t += hexdigit_to_int(*hex) * i;
 				} else {
 					zc_error_prev_line(
@@ -388,7 +388,6 @@ zparser_conv_ilnp64(region_type *region, const char *text)
 	int ngroups, num;
 	unsigned long hex;
 	const char *ch;
-	int c;
 	char digits[ILNP_MAXDIGITS+1];
 	unsigned int ui[ILNP_NUMGROUPS];
 	uint16_t a[ILNP_NUMGROUPS];
@@ -415,10 +414,9 @@ zparser_conv_ilnp64(region_type *region, const char *text)
 		} else {
 			/* Our grammar is stricter than the one accepted by
 			 * strtol. */
-			c = (int) *ch;
-			if (!isxdigit(c)) {
+			if (!isxdigit((unsigned char)*ch)) {
 				zc_error_prev_line("ilnp64: invalid "
-					"(non-hexadecimal) character %c", c);
+					"(non-hexadecimal) character %c", *ch);
 				return NULL;
 			}
 			if (num >= ILNP_MAXDIGITS) {
@@ -510,12 +508,11 @@ zparser_conv_eui(region_type *region, const char *text, size_t len)
 	nnum = len/8;
 	num = 1;
 	for (ch = text; *ch != '\0'; ch++) {
-		int c = (int) *ch;
 		if (*ch == '-') {
 			num++;
-		} else if (!isxdigit(c)) {
+		} else if (!isxdigit((unsigned char)*ch)) {
 			zc_error_prev_line("eui%u: invalid (non-hexadecimal) "
-				"character %c", (unsigned) len, c);
+				"character %c", (unsigned) len, *ch);
 			return NULL;
 		}
 	}
@@ -590,7 +587,7 @@ zparser_conv_tag(region_type *region, const char *text, size_t len)
 		return NULL;
 	}
 	for (ptr = text; *ptr; ptr++) {
-		if (!isdigit(*ptr) && !islower(*ptr)) {
+		if (!isdigit((unsigned char)*ptr) && !islower((unsigned char)*ptr)) {
 			zc_error_prev_line("invalid tag %s: contains invalid char %c",
 				text, *ptr);
 			return NULL;
@@ -786,14 +783,14 @@ precsize_aton (char *cp, char **endptr)
 	int exponent;
 	int mantissa;
 
-	while (isdigit((int)*cp))
+	while (isdigit((unsigned char)*cp))
 		mval = mval * 10 + hexdigit_to_int(*cp++);
 
 	if (*cp == '.') {	/* centimeters */
 		cp++;
-		if (isdigit((int)*cp)) {
+		if (isdigit((unsigned char)*cp)) {
 			cmval = hexdigit_to_int(*cp++) * 10;
-			if (isdigit((int)*cp)) {
+			if (isdigit((unsigned char)*cp)) {
 				cmval += hexdigit_to_int(*cp++);
 			}
 		}
@@ -858,17 +855,17 @@ zparser_conv_loc(region_type *region, char *str)
 
 		if (!parse_int(str, &str, &deg, "degrees", 0, 180))
 			return NULL;
-		if (!isspace((int)*str)) {
+		if (!isspace((unsigned char)*str)) {
 			zc_error_prev_line("space expected after degrees");
 			return NULL;
 		}
 		++str;
 
 		/* Minutes? */
-		if (isdigit((int)*str)) {
+		if (isdigit((unsigned char)*str)) {
 			if (!parse_int(str, &str, &min, "minutes", 0, 60))
 				return NULL;
-			if (!isspace((int)*str)) {
+			if (!isspace((unsigned char)*str)) {
 				zc_error_prev_line("space expected after minutes");
 				return NULL;
 			}
@@ -876,7 +873,7 @@ zparser_conv_loc(region_type *region, char *str)
 		}
 
 		/* Seconds? */
-		if (isdigit((int)*str)) {
+		if (isdigit((unsigned char)*str)) {
 			start = str;
 			if (!parse_int(str, &str, &i, "seconds", 0, 60)) {
 				return NULL;
@@ -886,7 +883,7 @@ zparser_conv_loc(region_type *region, char *str)
 				return NULL;
 			}
 
-			if (!isspace((int)*str)) {
+			if (!isspace((unsigned char)*str)) {
 				zc_error_prev_line("space expected after seconds");
 				return NULL;
 			}
@@ -929,7 +926,7 @@ zparser_conv_loc(region_type *region, char *str)
 		if (lat != 0 && lon != 0)
 			break;
 
-		if (!isspace((int)*str)) {
+		if (!isspace((unsigned char)*str)) {
 			zc_error_prev_line("space expected after latitude/longitude");
 			return NULL;
 		}
@@ -942,7 +939,7 @@ zparser_conv_loc(region_type *region, char *str)
 		return NULL;
 	}
 
-	if (!isspace((int)*str)) {
+	if (!isspace((unsigned char)*str)) {
 		zc_error_prev_line("space expected before altitude");
 		return NULL;
 	}
@@ -966,7 +963,7 @@ zparser_conv_loc(region_type *region, char *str)
 		if (!parse_int(str + 1, &str, &i, "altitude fraction", 0, 99)) {
 			return NULL;
 		}
-		if (!isspace((int)*str) && *str != '\0' && *str != 'm') {
+		if (!isspace((unsigned char)*str) && *str != '\0' && *str != 'm') {
 			zc_error_prev_line("altitude fraction must be a number");
 			return NULL;
 		}
@@ -975,7 +972,7 @@ zparser_conv_loc(region_type *region, char *str)
 		zc_error_prev_line("altitude must be expressed in meters");
 		return NULL;
 	}
-	if (!isspace((int)*str) && *str != '\0')
+	if (!isspace((unsigned char)*str) && *str != '\0')
 		++str;
 
 	if (sscanf(start, "%lf", &d) != 1) {
@@ -984,16 +981,16 @@ zparser_conv_loc(region_type *region, char *str)
 
 	alt = (uint32_t) (10000000.0 + d * 100 + 0.5);
 
-	if (!isspace((int)*str) && *str != '\0') {
+	if (!isspace((unsigned char)*str) && *str != '\0') {
 		zc_error_prev_line("unexpected character after altitude");
 		return NULL;
 	}
 
 	/* Now parse size, horizontal precision and vertical precision if any */
-	for(i = 1; isspace((int)*str) && i <= 3; i++) {
+	for(i = 1; isspace((unsigned char)*str) && i <= 3; i++) {
 		vszhpvp[i] = precsize_aton(str + 1, &str);
 
-		if (!isspace((int)*str) && *str != '\0') {
+		if (!isspace((unsigned char)*str) && *str != '\0') {
 			zc_error_prev_line("invalid size or precision");
 			return NULL;
 		}
@@ -1153,6 +1150,10 @@ void
 zadd_rdata_txt_wireformat(uint16_t *data, int first)
 {
 	rdata_atom_type *rd;
+	if (parser->current_rr.rdata_count >= MAXRDATALEN) {
+		zc_error_prev_line("too many rdata txt elements");
+		return;
+	}
 	
 	/* First STR in str_seq, allocate 65K in first unused rdata
 	 * else find last used rdata */
@@ -1186,8 +1187,10 @@ zadd_rdata_txt_clean_wireformat()
 {
 	uint16_t *tmp_data;
 	rdata_atom_type *rd = &parser->current_rr.rdatas[parser->current_rr.rdata_count-1];
+	if(!rd || !rd->data)
+		return; /* previous syntax failure */
 	if ((tmp_data = (uint16_t *) region_alloc(parser->region, 
-		rd->data[0] + 2)) != NULL) {
+		((size_t)rd->data[0]) + ((size_t)2))) != NULL) {
 		memcpy(tmp_data, rd->data, rd->data[0] + 2);
 		/* rd->data of u16+65535 freed when rr_region is freed */
 		rd->data = tmp_data;
@@ -1445,11 +1448,15 @@ process_rr(void)
 		if (i < rrset->rr_count) {
 			return 0;
 		}
+		if(rrset->rr_count == 65535) {
+			zc_error_prev_line("too may RRs for domain RRset");
+			return 0;
+		}
 
 		/* Add it... */
 		o = rrset->rrs;
-		rrset->rrs = (rr_type *) region_alloc(parser->region,
-			(rrset->rr_count + 1) * sizeof(rr_type));
+		rrset->rrs = (rr_type *) region_alloc_array(parser->region,
+			(rrset->rr_count + 1), sizeof(rr_type));
 		memcpy(rrset->rrs, o, (rrset->rr_count) * sizeof(rr_type));
 		region_recycle(parser->region, o,
 			(rrset->rr_count) * sizeof(rr_type));
@@ -1588,7 +1595,8 @@ zonec_read(const char* name, const char* zonefile, zone_type* zone)
 	yyparse();
 
 	/* remove origin if it was unused */
-	domain_table_deldomain(parser->db, parser->origin);
+	if(parser->origin != error_domain)
+		domain_table_deldomain(parser->db, parser->origin);
 	/* rr_region has been emptied by now */
 	dname = dname_parse(parser->rr_region, name);
 
@@ -1694,7 +1702,8 @@ zonec_parse_string(region_type* region, domain_table_type* domains,
 		*parsed = NULL;
 	else	*parsed = parser->prev_dname;
 	/* remove origin if it was not used during the parse */
-	domain_table_deldomain(parser->db, parser->origin);
+	if(parser->origin != error_domain)
+		domain_table_deldomain(parser->db, parser->origin);
 	zonec_desetup_string_parser();
 	return errors;
 }
