@@ -1,8 +1,8 @@
-/* A Bison parser, made by GNU Bison 2.5.  */
+/* A Bison parser, made by GNU Bison 2.7.  */
 
 /* Bison interface for Yacc-like parsers in C
    
-      Copyright (C) 1984, 1989-1990, 2000-2011 Free Software Foundation, Inc.
+      Copyright (C) 1984, 1989-1990, 2000-2012 Free Software Foundation, Inc.
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,6 +30,15 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
+#ifndef YY_YY_ZPARSER_H_INCLUDED
+# define YY_YY_ZPARSER_H_INCLUDED
+/* Enabling traces.  */
+#ifndef YYDEBUG
+# define YYDEBUG 0
+#endif
+#if YYDEBUG
+extern int yydebug;
+#endif
 
 /* Tokens.  */
 #ifndef YYTOKENTYPE
@@ -107,17 +116,20 @@
      T_LP = 325,
      T_EUI48 = 326,
      T_EUI64 = 327,
-     DOLLAR_TTL = 328,
-     DOLLAR_ORIGIN = 329,
-     NL = 330,
-     SP = 331,
-     STR = 332,
-     PREV = 333,
-     BITLAB = 334,
-     T_TTL = 335,
-     T_RRCLASS = 336,
-     URR = 337,
-     T_UTYPE = 338
+     T_CAA = 328,
+     T_CDS = 329,
+     T_CDNSKEY = 330,
+     DOLLAR_TTL = 331,
+     DOLLAR_ORIGIN = 332,
+     NL = 333,
+     SP = 334,
+     STR = 335,
+     PREV = 336,
+     BITLAB = 337,
+     T_TTL = 338,
+     T_RRCLASS = 339,
+     URR = 340,
+     T_UTYPE = 341
    };
 #endif
 /* Tokens.  */
@@ -191,26 +203,27 @@
 #define T_LP 325
 #define T_EUI48 326
 #define T_EUI64 327
-#define DOLLAR_TTL 328
-#define DOLLAR_ORIGIN 329
-#define NL 330
-#define SP 331
-#define STR 332
-#define PREV 333
-#define BITLAB 334
-#define T_TTL 335
-#define T_RRCLASS 336
-#define URR 337
-#define T_UTYPE 338
-
+#define T_CAA 328
+#define T_CDS 329
+#define T_CDNSKEY 330
+#define DOLLAR_TTL 331
+#define DOLLAR_ORIGIN 332
+#define NL 333
+#define SP 334
+#define STR 335
+#define PREV 336
+#define BITLAB 337
+#define T_TTL 338
+#define T_RRCLASS 339
+#define URR 340
+#define T_UTYPE 341
 
 
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
 {
-
-/* Line 2068 of yacc.c  */
+/* Line 2058 of yacc.c  */
 #line 50 "zparser.y"
 
 	domain_type	 *domain;
@@ -222,9 +235,8 @@ typedef union YYSTYPE
 	uint16_t	 *unknown;
 
 
-
-/* Line 2068 of yacc.c  */
-#line 228 "zparser.h"
+/* Line 2058 of yacc.c  */
+#line 240 "zparser.h"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
@@ -233,4 +245,18 @@ typedef union YYSTYPE
 
 extern YYSTYPE yylval;
 
+#ifdef YYPARSE_PARAM
+#if defined __STDC__ || defined __cplusplus
+int yyparse (void *YYPARSE_PARAM);
+#else
+int yyparse ();
+#endif
+#else /* ! YYPARSE_PARAM */
+#if defined __STDC__ || defined __cplusplus
+int yyparse (void);
+#else
+int yyparse ();
+#endif
+#endif /* ! YYPARSE_PARAM */
 
+#endif /* !YY_YY_ZPARSER_H_INCLUDED  */
